#pragma once

#include "common.hpp"
#include "handleCache.hpp"
#include "serverSideLink.hpp"
#include "storage.hpp"

#include <folly/Bits.h>
#include <folly/SpinLock.h>
#include <folly/futures/Future.h>
#include <folly/io/IOBuf.h>
#include <folly/io/IOBufQueue.h>
#include <monitoring/monitoring.h>

#include <cstdint>
#include <memory>

namespace rtransfer {

class Reader {
public:
    Reader(StoragesMap &storages, HandleCache &handleCache,
        std::shared_ptr<ServerSideLink> serverLink);

    folly::Future<std::size_t> read(std::uint64_t reqId,
        folly::fbstring storageId, folly::fbstring fileId,
        folly::fbstring fileGuid, std::uint64_t offset, std::size_t size,
        std::uint8_t priority);

private:
    static std::unique_ptr<folly::IOBuf> constructDataMsg(std::uint64_t reqId,
        std::uint64_t offset, std::uint8_t part, bool isLastPart,
        std::unique_ptr<folly::IOBuf> buf);

    std::vector<folly::Future<folly::Unit>> sendParts(folly::IOBufQueue buf,
        std::uint64_t reqId, std::uint64_t offset,
        std::chrono::steady_clock::time_point now);

    StoragesMap &storages_;
    HandleCache &handleCache_;
    std::shared_ptr<ServerSideLink> serverLink_;
};

}  // namespace rtransfer
